#include "FileVIDStereoDevice.h"
#include <sys/types.h>
#include <iostream>
#include <dirent.h>


#define RGB2Y(r, g, b, y) \
  y = (306*(r) + 601*(g) + 117*(b)) >> 10; \
  y = y < 0 ? 0 : y;\
  y = y > 255 ? 255 : y;


FileVIDStereoDevice::FileVIDStereoDevice()
{
	is_timestamp=false;
	fl=fr=fc=NULL; fvid=NULL;
	videosource = new svsStoredImages();
}


FileVIDStereoDevice::~FileVIDStereoDevice()
{
	if (fl) { free(fl); fl=NULL; }
	if (fr) { free(fr); fr=NULL; }
	if (fc) { free(fc); fc=NULL; }
	delete videosource;
}

bool FileVIDStereoDevice::init()
{
    return true;
}


void FileVIDStereoDevice::dirname()
{
  strcpy(dir,filebasename);
  char *p = strstr(dir,"/");
  char *lastp = NULL;
  while (p!=NULL) {
    lastp = p;
    p = strstr(p+1,"/");
  }
  if (lastp==NULL)
    strcpy(dir,".");
  else
    *lastp ='\0';
}


void FileVIDStereoDevice::filenames(const char* filename) {
     struct dirent **direct;
     int n = scandir(dir, &direct, 0, alphasort);
     if( n  == 0) {
	printf("Can't open other files in the current directory");
     } else {
	for(int i = 0; i < n; ++i)
		if(strstr(direct[i]->d_name,".vid")) {
			  string s = (string) dir;
			  s += "/" + string(direct[i]->d_name);
			  //if(contr)
			  if(strcmp(filename, s.c_str()) != 0)
			      files.push_back(s);
			  //if(strcmp(filename, s.c_str()) == 0)
			  //  contr = true;
		}
			
     }

/*    DIR  *d;
    struct dirent *direct;
    files = vector<string>();
    d = opendir(dir);
    bool contr = false;
    if( d == NULL ) {
      printf("Can't open other files in the current directory");
    }
    while ((direct = readdir(d)) != NULL) 
	if(strstr(direct->d_name,".vid")&&!strstr(direct->d_name,".ini")) {
	  string s = (string) dir;
	  s += "/" + string(direct->d_name);
	  if(contr)
	      files.push_back(s);
	  if(strcmp(filename, s.c_str()) == 0)
	    contr = true;
	}
   
    closedir(d);*/
}


void FileVIDStereoDevice::init(const char* filename, const char* fileIniName, bool rect)
{ 
  fvid = fopen(filename, "r");
  if (!fvid) {
    printf("Error in opening VID file %s.\n",filename);
    return;
  }
  
  FILE *fini = fopen(fileIniName, "r");
  if(!fini) {
      printf("Error in opening VID file %s.\n",fileIniName);
       return;
  }
  fclose(fini);
  
  filebasename = (char*)filename;
  dirname();
  filenames(filename);
  
  
  char line[80]; nfields=0;
  while (fgets(line,80,fvid)) {
  	  if (line[0]=='*') break;
  	  strcpy(field[nfields],line); nfields++;
  }

	width=0; height=0;
  for (int k=0; k<nfields; k++) {
  	printf("FIELD %d: %s\n",k,field[k]);
  	if (strncasecmp(field[k],"timestamp",9)==0) {
    	is_timestamp=true;
    }
    if (strstr(field[k],"RGB32") || strstr(field[k],"GRAY8")) {
  	    char buf1[32];
  		sscanf( field[k], "%s %d %d", buf1, &width, &height );
    }
  }
  
    printf("Reading VID file %d %d %s\n", width, height, (is_timestamp?"+TS":""));
  
  if (width <= 0 || height <= 0 ) {
    printf("Error in opening %s: BAD FORMAT.\n",filename);
    return;
  }

  printf("Reading VID file %d %d %s\n", width, height, (is_timestamp?"+TS":""));

  // raw video data [32bpp left color + 8bit right] * nframes
  
  // memory allocation
  fl = (unsigned char*)malloc(width*height);   // 8 bit image
  fr = (unsigned char*)malloc(width*height);   // 8 bit image
  fc = (unsigned char*)malloc(4*width*height); // 32 bit color image RGBX   SVS wants this format!!!
  printf("  ** %d bytes allocated for frame_left/right **\n", width*height);

    stereoimage.frame_left = fl;
    stereoimage.frame_right = fr;
    stereoimage.frame_color_left = fc;
    stereoimage.frame_width = width; stereoimage.frame_height = height;
    stereoimage.leftColorImage.create(height, width, CV_8UC3);
    stereoimage.rightColorImage.create(height, width, CV_8UC3);
    stereoimage.leftGrayImage.create(height, width, CV_8UC1);
    stereoimage.rightGrayImage.create(height, width, CV_8UC1);

    disparityImage.create(height, width, CV_8UC3);
    //stereoimage.bpp = 4;


  //char SVSinifile[250];
  
  //sprintf(SVSinifile,"%s.ini",filename);
  bool ret = videosource->ReadParams((char*)fileIniName);
  if (!ret)
    printf("      Warning: cannot open SVS parameters file %s\n",fileIniName);
  else
    printf("      Read SVS parameters %s.\n",fileIniName);

  videosource->SetRect(rect);
  if (rect)
    printf("      Image Rectification enabled.\n");
  else
    printf("      Image Rectification disabled.\n");

  return;


}

bool FileVIDStereoDevice::ready()
{ return fvid!=NULL  && !feof(fvid); }


bool FileVIDStereoDevice::OthersFiles()
{
  if(files.size() == 0)
    return false;
  else{
    fclose(fvid); //chiudo il precedente file
    fvid = fopen(files[0].c_str(), "r");
    while (!fvid) {
      if(files.size() == 0)
	return false;
      printf("Error in opening VID file %s.\n",files[0].c_str());
      files.erase(files.begin()); //cancello il nome del files che non si apre
      fvid = fopen(files[0].c_str(), "r"); //passo all'altro elemento
    }
    printf("Analysing Video: %s. \n", files[0].c_str());
    files.erase(files.begin()); //cancello l'elemento dal vector*/
    contr2 = true;
  }
  return true;
}


bool FileVIDStereoDevice::getFrame()
{    
      if (feof(fvid))
    	  return false;
 

	for (int k=0; k<nfields; k++) {
  		// printf("READ FIELD %d: %s\n",k,field[k]);
  		if (strstr(field[k],"TIMESTAMP")) {
			char buf[160]; long l;
			fgets(buf,160,fvid);
			sscanf(buf,"%lu",&l);
			this->timestamp = l;
			// printf("R [%lu]\n",this->frame_timestamp);  			
  		}
  		else if (strstr(field[k],"RGB32")) { // followed by GRAY8
		    // reads color left and grayscale right
		    int r = fread(fc, width * height * 4, 1, fvid) *
		            fread(fr, width * height, 1, fvid);
		    if (r<1)
		      return OthersFiles();
		    k++; // skip next field
		    // color left to grayscale left
		    for (int i=0; i<width*height; i++) {
		      int y;
		      RGB2Y((fc)[i*4],(fc)[i*4+1],(fc)[i*4+2],y);
		      (fl)[i] = (unsigned char)(y);
		    }
  		}
  		else if (strstr(field[k],"GRAY8")) { // followed by GRAY8
		    // reads color left and grayscale
		    int r = fread(fl, width * height, 1, fvid) *
		            fread(fr, width * height, 1, fvid);
	    
		    k++; // skip next field
		    // grayscale left to color left
		    for (int i=0; i<width*height; i++) {
			(fc)[i*4]=(fc)[i*4+1]=(fc)[i*4+2]=(fl)[i];
		    }
  		}
  		else if (strstr(field[k],"XSDATA")) { 
		    // Reads bytes from xsens
		    fread(xsdata,sizeof(float),NXSDATA,fvid);
  		}
	}



    // load images in memory 
    videosource->Load(width, height, fl, fr, fc, NULL, false, true);
    // videosource->SetRect(rect);
    svsstereoimage =  videosource->GetImage(100);

   int indexRGBX = 0;
	for(int i = 0; i < height; i++)
	{
		for(int j = 0; j < width; j++)
		{
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[0] = stereoimage.frame_color_left[indexRGBX+2];
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[1] = stereoimage.frame_color_left[indexRGBX+1];
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[2] = stereoimage.frame_color_left[indexRGBX];


			indexRGBX += 4;

		}
	}
  
    return true;

}

void FileVIDStereoDevice::done()
{
	if (fvid) {
		fclose(fvid);
		fvid=NULL;	
	}

}


void FileVIDStereoDevice::getXSdata(float *f)
{
	for (int k=0; k<NXSDATA; k++)
		f[k]=xsdata[k];
}
