#ifndef __STEREODEVICE_H__
#define __STEREODEVICE_H__

#include "CameraDevice.h"
#include "svsclass.h"

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

struct colorRange {
	int min, max;
	cv::Vec3b color;
};

struct colorTable {
	colorRange colors[8];
};

struct StereoImage {
  int frame_width, frame_height;
  unsigned char *frame_left;
  unsigned char *frame_right;
  unsigned char *frame_color_left;   // 32bpp RGBX  (this is the format SVS uses)
  unsigned char *frame_color_right;   // 32bpp RGBX  (this is the format SVS uses)

	cv::Mat leftColorImage;
	cv::Mat rightColorImage; 
	cv::Mat leftGrayImage;
	cv::Mat rightGrayImage;
};

class StereoDevice {
public:
	int width, height;
	StereoImage stereoimage;
	colorTable tableColor;
	unsigned long timestamp;
  	svsStereoImage *svsstereoimage;
  	short *disparity;
	char cameraid[33]; 
	bool norect;

	cv::Mat disparityImage;

	StereoDevice();
	virtual ~StereoDevice() { } 
 	virtual bool init() = 0;
	virtual bool ready() = 0;
	virtual bool getFrame() = 0;
	virtual void done() { }
	bool calcDisparity();
	cv::Mat processDisparity();
	virtual bool calc3D(int x, int y, double *X, double *Y, double *Z);
	
};

#endif
