//////////////////////////////////////////////////////////////////////
//
// stereo.cpp : Stereo acquisition and computation
// 2005 by Luca Iocchi
//
//////////////////////////////////////////////////////////////////////

#include <math.h>
#include <stdlib.h>
#include <unistd.h>

#include "VidereStereoDevice.h"

#define RGB2Y(r, g, b, y) \
  y = (306*(r) + 601*(g) + 117*(b)) >> 10; \
  y = y < 0 ? 0 : y;\
  y = y > 255 ? 255 : y;


VidereStereoDevice::VidereStereoDevice()
{
  videosource=NULL; 
  svsstereoimage=NULL;
  cameraid[0]='\0';
  width=640; height=480;	
}

VidereStereoDevice::~VidereStereoDevice()
{
	//if (videosource)
	//	delete videosource;
}

bool VidereStereoDevice::init()
{
	return init(width,height,15);
}

bool VidereStereoDevice::init(int w, int h, int fps, bool rect)
{
	width=w; height=h;
	
  videosource = getVideoObject(); 
  printf("      %s\n", svsVideoIdent);

  int nc = videosource->Enumerate();
  char** dev = videosource->DeviceIDs();

  if (nc==0) {
    printf("      No cameras found!\n");
    return false;
  }

  for (int i=0; i<nc; i++) {
    printf("         %s\n", dev[i]);  // Valgrind:: dev contains uninitialised or 
                                      // unaddressable byte(s) | suppressed
  }

  int l = strlen(dev[0]);
  strncpy(cameraid,(dev[0]+l-4),4);    // Valgrind error | suppressed
  cameraid[4]='\0';
  printf("         Camera ID: %s\n", cameraid);

  char SVSinifile[80];
  sprintf(SVSinifile,"svs-%s.ini", cameraid);
  bool ret = videosource->ReadParams(SVSinifile);
  if (!ret)
    printf("      Warning: cannot open SVS parameters file %s\n",SVSinifile);
  else
    printf("      Read SVS parameters %s.\n",SVSinifile);


  ret = videosource->Open();
  if (!ret) {
    printf("*** Error. Cannot open video device. ***\n");
    return false;
  }

  printf("      Video device opened.\n");
  printf("      Image size %dx%d.\n",width,height);   

  videosource->SetSize(width,height);
     
  videosource->SetRate(fps);   

  videosource->SetColor(true,true);
    
  // set this into svs.ini file (DOES NOT WORK!!!)
  videosource->use_autogain = false;            // true if using autogain
  videosource->use_autoexposure = false;	// true if using autoexposure
  //videosource->use_autowhite = false;           // true if using autowhite
  //videosource->use_autobrightness = false;      // true if using autobrightness
  
  
  
  
  printf("      Automatic gain:%d exposure:%d white:%d brightness:%d\n",
	 videosource->use_autogain, videosource->use_autoexposure,
	 videosource->use_autowhite, videosource->use_autobrightness);
 
 
 printf("      Gain:%d exposure:%d red:%d blue:%d\n",
	 videosource->gain, videosource->exposure,
	 videosource->red, videosource->blue);
 
  videosource->color_alg = COLOR_ALG_BEST;      // [COLOR_ALG_FAST, COLOR_ALG_BEST]

  videosource->SetRect(rect);
  if (rect) 
    printf("      Image Rectification enabled.\n");
  else
    printf("      Image Rectification disabled.\n");

  ret = videosource->Start();
  if (!ret)   {
    printf("      Can't start continuous capture\n");
    return false;
  }

	stereoimage.leftColorImage.create(height, width, CV_8UC3);
	stereoimage.rightColorImage.create(height, width, CV_8UC3); 
	stereoimage.leftGrayImage.create(height, width, CV_8UC1);
	stereoimage.rightGrayImage.create(height, width, CV_8UC1);


	disparityImage.create(height, width, CV_8UC3);

  return true;

}

bool VidereStereoDevice::ready()
{
	return true;
}

bool VidereStereoDevice::getFrame()
{
	svsstereoimage = videosource->GetImage(400);
	if (svsstereoimage==NULL || !svsstereoimage->haveImages) {
    	printf("No images from device\n");
     	//no_images_count ++;
     	//if (no_images_count%5==0)
     	//init_stereo(basefilename);
     	return false;
  	}

  	stereoimage.frame_left = svsstereoimage->Left();
  	stereoimage.frame_right = svsstereoimage->Right();

  	if (svsstereoimage->haveColor)
	{
    		stereoimage.frame_color_left = (unsigned char*)svsstereoimage->Color();
        }
	else
	{
   		stereoimage.frame_color_left = NULL;
	}
	if(svsstereoimage->haveColorRight)
	{
		stereoimage.frame_color_right = (unsigned char*)svsstereoimage->ColorRight();
	}
	else
	{
		stereoimage.frame_color_right = NULL;
	}
  	stereoimage.frame_width  = svsstereoimage->ip.width;
  	stereoimage.frame_height = svsstereoimage->ip.height;

	timestamp = getTimeStamp();


	int indexRGBX = 0;
	for(int i = 0; i < height; i++)
	{ 
		for(int j = 0; j < width; j++)
		{
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[0] = stereoimage.frame_color_left[indexRGBX+2];
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[1] = stereoimage.frame_color_left[indexRGBX+1];
			stereoimage.leftColorImage.at<cv::Vec3b>(i,j)[2] = stereoimage.frame_color_left[indexRGBX];

			stereoimage.rightColorImage.at<cv::Vec3b>(i,j)[0] = stereoimage.frame_color_right[indexRGBX+2];
			stereoimage.rightColorImage.at<cv::Vec3b>(i,j)[1] = stereoimage.frame_color_right[indexRGBX+1];
			stereoimage.rightColorImage.at<cv::Vec3b>(i,j)[2] = stereoimage.frame_color_right[indexRGBX];
			
			indexRGBX += 4;
			
		}
	}

	return true;
}


void VidereStereoDevice::done()
{
	videosource->Close();	
}


