/*
 *  imbs.h
 *
 *  Author: Domenico Bloisi
 *  domenicobloisi@yahoo.com
 *  Copyright 2010
 */

#ifndef __IMBS_H_
#define __IMBS_H_

#include <fstream>
#include <sstream>
#include "grabber.h"
#include "BlobResult.h"
#include "math.h"
#include "findBall.h"

#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/legacy/compat.hpp>
#include <opencv2/imgproc/imgproc_c.h>

class IMBS {

public:
	typedef struct {
		uchar r;
		uchar g;
		uchar b;
        int16_t d;
	} Pixel;

	typedef struct {
		Pixel* currentValue;
		char* counter;
	} BgStruct;

	typedef struct {
		Pixel* values;
		bool* isValid;
		int* counter;
	} BgModel;

	//SHADOW SUPPRESSION PARAMETERS
	double alpha;
	double beta;
	double tau_s;
	double tau_h;
	Grabber *grabber;

private:
    IMBS *stereoImbs;

	int frameWidth, frameHeight;

	int numPixels;

	IplImage* frame;
	cv::Mat frameStereo;
	short *disparity;

	IplImage* bgSample;

	BgStruct* bgValues;
    BgStruct* prevBgValues;

	BgModel* bgModel;

	int totalImageParts;

	int minBinHeight;

	int numSamples;

	unsigned int samplingPeriod;

	unsigned long prev_bg_frame_time;

	int bg_frame_counter;

	int image_part_index;

	int binWidth;

	int maxBgValues;

	bool isCreated;
	string* bgFilename;
    string bgStereoFilename;

	IplImage* hsvMat;
	IplImage* rgbMat;

	double area;

	bool bg_reset;

    bool prev_area;

	bool stereo;

	short *fgStereo;
    short *currentDisparity;

	IplImage* grayForeground;
	IplImage* whiteForeground;

public:

	int fgThreshold;

	IplImage* bgImage;
	IplImage* bgStereoImage;

    cv::Mat bgStereoImageMat;

	IplImage* hsvFrame;

	IplImage* fgImage;
	IplImage* fgFilteredImage;
	IplImage* fgColoredImage;

    IplImage *depthBlobs;

	IMBS(Grabber *g,
			int fgThreshold,
			int binWidth,
			unsigned long samplingPeriod,
			int totalImageParts,
			int minBinHeight,
			int	numSamples,
			double alpha,
			double beta,
			double tau_s,
			double tau_h,
			double area,
			IMBS *stereoImbs = NULL);

	IMBS(Grabber *g,
				int fgThreshold,
				int binWidth,
				unsigned long samplingPeriod,
				int totalImageParts,
				int minBinHeight,
				int	numSamples);


	~IMBS();
	void init();
	int getFrameWidth(){return frameWidth;};
	int getFrameHeight(){return frameHeight;};
	IplImage *getFrame(){return frame;};
    void process();
	void processStereo();
	int getMaxBgValues(){
		return maxBgValues;
	}
	BgModel* getBgModel(){
		return bgModel;
	}
	short * getDisparity(){return disparity;};
	bool loadBackground(const char* filename);
	bool loadStereoBackground(const char* filename);
	void setSaveBackground(string* filename);
    void setSaveStereoBackground(string filename);
	CBlobResult getBlobForeground();

private:
	void getForeground();
    void analyzeGray();
	void hsvSuppression();
	void updateBackground();
	void createBackground(int frame_number, int image_interval, bool do_background);
	void createStereoBackground(int frame_number, int image_interval, bool do_background);
	void areaThresholding(double area);
	void refineForeground();
	void getStereoForeground();
    void heightFilter();
    void depthFilter();
	bool checkBlobArea(CBlobResult playerBlobs, double area);
};

#endif // __IMBS_H_
