#ifndef KINECTGRABBER_H
#define KINECTGRABBER_H

#include "grabber.h"
#include <string>
#include <sstream>
#include <opencv2/opencv.hpp>
#include <fstream>
#include "Blob.h"
#include "BlobResult.h"
#include "myfreenectdevice.h"

class kinectgrabber : public Grabber
{
    public:
        kinectgrabber(const int width, const int height, const int kinect_number, const string ffn, int fps=25, int prev_offset=1);
        virtual ~kinectgrabber();
        int getWidth(){return width;};
        int getHeight(){return height;};
        cv::Mat getDepth() { return depth8bit; };
        IplImage *getImage() { return frame; };
        cv::Mat getDepth16bit();
        void get3DPoint(const int x, const int y, double *wx, double *wy, double *wz);
        virtual unsigned long getTimestamp();
        virtual unsigned long getPrevTimestamp();
        virtual bool getData();
        double getFps(){return fps;};
        void reset();
    private:
        double time_scale;
        int width, height;
        int prev_offset;
        int lx, ly, rx, ry;
        cv::Mat homography;
        double ox, oy, oz;
        double theta, translation, h;
        std::string homographyFilename;
        MyFreenectDevice* device;
        Freenect::Freenect freenect;
        cv::Mat depth8bit;
        cv::Mat depth16bitNew;
        cv::Mat frameNew;
        cv::Mat calcColorDepth();
};

#endif // KINECTGRABBER_H
