/*
 *  livegrabber.cpp
 *
 *  Author: Domenico Bloisi
 *  domenicobloisi@yahoo.com
 *  Copyright 2010
 */

#include "livegrabber.h"

LiveGrabber::LiveGrabber(int camera_index, int prev_offset) {
	prev_counter = 0;
	this->prev_offset = prev_offset;

	capture = cvCaptureFromCAM(camera_index);
	if(!capture) {
		cout<<"unable to capture from camera."<<endl;
		cout<<"aborting..."<<endl;
		abort();
	}

	cvQueryFrame(capture);
	height = (int) cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_HEIGHT);
	width = (int) cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH);
	fps = -1;
	CvSize frameSize = cvSize(width, height);

	frameNumber = 0;

	frame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);
	prevFrame = cvCreateImage(frameSize, IPL_DEPTH_8U, 3);

	timestamp = getTimestamp();
}

LiveGrabber::~LiveGrabber()
{
	cvReleaseImage(&frame);
	cvReleaseImage(&prevFrame);
	cvReleaseCapture(&capture);
}

bool LiveGrabber::getData()
{
	prev_counter++;
	if(prev_counter == prev_offset)
	{
		cvCopy(frame, prevFrame);
		prev_counter = 0;
	}

	if(!cvGrabFrame( capture )) {
		return false;
	}
	cvCopy(cvRetrieveFrame(capture), frame);

	prevTimestamp = timestamp;
	timestamp = getTimestamp();

	frameNumber++;
	return true;
}

unsigned long LiveGrabber::getTimestamp() {
#ifdef __linux__
        struct timeval tv;
        gettimeofday(&tv,0);
        return (unsigned long)(tv.tv_sec%86400)*1000+(unsigned long)tv.tv_usec/1000;
#else
        struct timeb timebuffer;
        ftime( &timebuffer );
        unsigned short  millitm = timebuffer.millitm;
        time_t sectm = timebuffer.time;
        return (unsigned long)(sectm%86400)*1000+(unsigned long)millitm;
#endif
}

unsigned long LiveGrabber::getPrevTimestamp() {
	return prevTimestamp;
}
