/*
 *  main.cpp
 *
 *  Author: Domenico Bloisi
 *  domenicobloisi@yahoo.com
 *  Copyright 2010
 */

#include "filegrabber.h"
#include "livegrabber.h"
#include "imagegrabber.h"
#include "imbs.h"
#include "imagestereograbber.h"
#include "kinectgrabber.h"

#include <sstream>
#include <ctype.h>
#include <dirent.h>
#include <iostream>

#include <stdio.h>
#ifdef __linux__
	#include <sys/stat.h>
	#include <sys/types.h>
 #endif

void createDirectory(string name);

unsigned long getTime();

//THREAD
//Scene3d scene3D;
bool startScene3d = false;
//boost::mutex mtx;

double planViewTranslation, planViewScale;
//svs3Dpoint adjustedPts3D[640 * 480];
Grabber *grabber;


void onMouse(int event, int x, int y, int flag, void *param);
IplImage* imageMosaic(int rows, int cols, int width, int height, IplImage** images);
bool loadPositionFile(const char *positionfile, double* degreesX, double* degreesZ, double* Yoffset, double* planViewTranslation, double *planViewScale);


static bool sort_using_greater_than(double u, double v)
{
   return u > v;
}

//MAIN
int main(int argc, char **argv) {
	//IMAGE GRABBING
//	Grabber *grabber;
	unsigned long lastTimeout = 0;

	//INPUT FROM VIDEO FILE
	string invideo;
    string fini;
    string camera_position;
	bool video_input = false;

	//OUTPUT: fg images
    bool save_fg = false;
    string save_fg_path;

    //OUTPUT: mosaic
    bool video_output = false;
    string outvideo;
    CvVideoWriter *writer = NULL;
    IplImage* images[4];
    IplImage* mosaic = NULL;
    //IplImage* greenField = NULL;
    int mosaic_width = 1024;
    int mosaic_height = 768;

    //INPUT FROM IMAGE SET
    string imagesetfile;
    bool image_input = false;
    bool fps_image_input = false;
    int fps_image = 25;

    //OPTIONS
	bool run = true;
	bool runStereo = true;
	bool slow = false;
	bool verbose = false;

	//IMBS PARAMETERS
    int fgThreshold = 6;
	int binWidth = 5;
	unsigned long samplingPeriod = 750;
	int totalImageParts = 1;
	int minBinHeight = 2;
	int	numSamples = 20;
	double alpha = 0.6;
	double beta = 1.15;
	double tau_s = .3;
	double tau_h = 40;
	double area = 500;

	//CAMERA POSITION PARAMETERS
	double degreesX, degreesZ, Yoffset;
	//double planViewTranslation, planViewScale;
	char positionfile[256];
	double cx, sx, cz, sz;
	double radiantsX;
	double radiantsZ;

	//GUI
	IplImage *frame = NULL;
	bool show_gui = true;
	CvFont font;
	IplImage* message = NULL;

	//LOAD and SAVE BACKGROUND
	bool load_bg = false;
	string load_bg_filename;
	bool save_bg = false;
	string save_bg_filename;

	//STEREO
	bool enableRectification = true;
	bool stereo_input = false;
    bool stereo_img_input = false;
    std::string stereo_img_directory;
    std::string settingfilename;
    int kinect_number;
    bool stereo_kinect = false;

	//LOAD and SAVE STEREO BACKGROUND
	bool load_stereo_bg = false;
	string load_stereo_bg_filename;
	bool save_stereo_bg = false;
	string save_stereo_bg_filename;

	//DETECTING PLAYERS
	bool players = false;
	bool checkoffside = false;
	string configuration_file_player;
	int numBlobs;
//	vector<player*> Players;
//	vector<previousPosition> startPosition;
//	vector<previousPosition> lastPosition;
	cv::Mat offsideImg;
	offsideImg = cv::imread("offside.png", CV_LOAD_IMAGE_COLOR);

//	svsStereoProcess *processObject = new svsStereoProcess();

	// ********** Process command line args *************************
	for (int i = 1; i < argc; i++) {
		if (strcmp(argv[i],"-vid") == 0) {
			video_input = true;
			invideo.assign(argv[++i]);
        }
		else if (strcmp(argv[i],"-stereo") == 0) {                        
			stereo_input = true;
            fini.assign(argv[++i]);
            camera_position.assign(argv[++i]);
		}
	 	else if (strcmp(argv[i],"-img") == 0) {
	 		image_input = true;
	 		imagesetfile.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-fps") == 0) {
			fps_image_input = true;
			fps_image = atoi(argv[++i]);
		}
	 	else if (strcmp(argv[i],"-slow") == 0) {
	 		slow = true;
	 	}
	 	else if (strcmp(argv[i],"-nogui") == 0) {
	 		show_gui = false;
	 	}
	 	else if (strcmp(argv[i],"-verbose") == 0) {
	 		verbose = true;
	 	}
	 	else if (strcmp(argv[i],"-out") == 0) {
	 		video_output = true;
	 		outvideo.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-savebg") == 0) {
	 		save_bg = true;
	 		save_bg_filename.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-loadbg") == 0) {
	 		load_bg = true;
	 		load_bg_filename.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-savestereobg") == 0) {
	 		save_stereo_bg = true;
	 		save_stereo_bg_filename.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-loadstereobg") == 0) {
	 		load_stereo_bg = true;
	 		load_stereo_bg_filename.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-savefg") == 0) {
	 		save_fg = true;
	 		save_fg_path.assign(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-players") == 0) {
			players = true;
			configuration_file_player.assign(argv[++i]);
		}
        else if (strcmp(argv[i],"-stereoimg") == 0) {
            stereo_img_input = true;
            stereo_img_directory.assign(argv[++i]);
            settingfilename.assign(argv[++i]);
            kinect_number = atoi(argv[++i]);
            std::cout << "kinect_number " << kinect_number << std::endl;
        } else if (strcmp(argv[i],"-kinect_live") == 0) {
            stereo_kinect = true;
            settingfilename.assign(argv[++i]);
            kinect_number = atoi(argv[++i]);
            std::cout << "kinect_number " << kinect_number << std::endl;
        }
	 	else if (strcmp(argv[i],"-h") == 0 || strcmp(argv[i],"-help") == 0) {
			cout<<"Options: -vid <filename>: input from video files (e.g., .avi files)\n"
                    "\t -stereoimg <directory> -file settings <filename> -kinect number <1,2,3,4>: input from a directory\n"
                    "\t -kinect_live file settings <filename> kinect number <1,2,3,4>: input from a kinect\n"
					"\t -img <first image name>: input from a list of images\n"
					"\t -players <position configuration filename>: detect three players in the environment \n"
					"\t  Notice that <first image name> must be in the form of \"img%d.jpg\"\n"
					"\t  where \"%d\" is the image number (e.g., \"img000.jpg\")\n"
					"\t -fps <fps value>: set the fps value (default value 25)\n"
					"\t -out <filename>: save the foreground images as a video\n"
					"\t -savefg <path>: save the current foreground image in <path>\n"
					"\t -slow: process one frame and wait\n"
					"\t -nogui: no graphical user interface will be shown\n"
					"\t -verbose\n"
					"\t -th <foreground threshold>: default value 6\n"
					"\t -bw <histogram bin width>: default value 5\n"
					"\t -p <sampling period in ms>: default value 1000\n"
					"\t -d <minimal bin height>: default value 2\n"
					"\t -n <number of samples>: default value 30\n"
					"\t -loadbg <filename>: load the prebuilt background model\n"
					"\t -savebg <filename>: save the initial background model\n"
					"\t -loadstereobg <filename>: load the prebuilt stereo background model\n"
					"\t -savestereobg <filename>: save the initial stereo background model\n"
					"\t -alpha <value>: shadow suppression parameter alpha (default value 0.75)\n"
					"\t -beta <value>: shadow suppression parameter beta (default value 1.15)\n"
					"\t -tau_s <value>: shadow suppression parameter tau_s (default value 30)\n"
					"\t -tau_h <value>: shadow suppression parameter tau_h (default value 20)\n"
					"\t -area <value>: filter out the observations smaller than <value> (default value 100)\n"
					"\t -stereo <filename>: enable stereo computation <filename> is the .ini file\n"
					"\t -norect: disable stereo rectification\n"
					"\t -h: print help"<<endl;
			return 0;
		}
	 	else if (strcmp(argv[i],"-th") == 0) {
	 		fgThreshold = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-bw") == 0) {
	 		binWidth = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-p") == 0) {
	 		samplingPeriod = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-tip") == 0) {
	 		totalImageParts = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-d") == 0) {
	 		minBinHeight = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-n") == 0) {
	 		numSamples = atoi(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-alpha") == 0) {
	 		alpha = atof(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-beta") == 0) {
	 		beta = atof(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-tau_s") == 0) {
	 		tau_s = atof(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-tau_h") == 0) {
	 		tau_h = atof(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-area") == 0) {
	 		area = atof(argv[++i]);
	 	}
	 	else if (strcmp(argv[i],"-norect")==0) {
	 		enableRectification = false;
	 	}
	 	else{
	 		cout<<endl;
	 		cout<<"Invalid input arguments."<<endl;
	 		cout<<"Usage: imbs [options]\nType \"imbs -h\" for the options list"<<endl;
	 		return 0;
	 	}
	}

//	if(!loadPositionFile(camera_position.c_str(), &degreesX, &degreesZ, &Yoffset, &planViewTranslation, &planViewScale)) {
//		cout << "unable to load file " << positionfile << endl;
//		return EXIT_FAILURE;
//	}
//	else {
//		cout << positionfile << " file loaded" << endl;
//	}

//	if(players) {
//		if(!loadPositionPlayersFile(configuration_file_player.c_str(), startPosition)) {
//			cout << "unable to load file " << configuration_file_player << endl;
//			return EXIT_FAILURE;
//		} else {
//			cout << configuration_file_player << " file loaded" << endl;
//		}
//	}

//	for(unsigned int i = 0; i < startPosition.size(); ++i) {
//		cout << "P" << i << ": " << startPosition[i].role << " " << startPosition[i].pos.x << " " << startPosition[i].pos.y << endl;
//	}

	radiantsX = degreesX * (M_PI/180.0);
	radiantsZ = degreesZ * (M_PI/180.0);
	cx = cos(radiantsX);
	sx = sin(radiantsX);
	cz = cos(radiantsZ);
	sz = sin(radiantsZ);

	//GLOBAL COORDINATES
//	svs3Dpoint adjustedPts3D[640 * 480];
        
    if(fini == "" && video_input) {
    	cout << "Error NO .INI FILE" << endl;
        return EXIT_FAILURE;
    }

	//control
	if(save_bg && load_bg) {
		cerr<<endl;
		cerr<<"ERROR: unable to set both -savebg and -loadbg."<<endl;
		cerr<<"Process terminated."<<endl;
		return EXIT_FAILURE;
	}
	if(image_input && video_input) {
		cerr<<endl;
		cerr<<"ERROR: two input sources selected."<<endl;
		cerr<<"Process terminated."<<endl;
		return EXIT_FAILURE;
	}

	// ********** END Process command line args *************************

//	if(stereo_input) {
//		if(video_input) {
//			cout<<endl;
//			cout<<"INPUT FROM STEREO VIDEO FILE"<<endl;
//			grabber = new StereoFileGrabber(invideo.c_str(), fini.c_str());
//			cout<<"frame dimension "<<grabber->width<<"x"<<grabber->height<<endl;
//			cout<<endl;
//		}
//		else {
//			cout<<endl;
//			cout<<"INPUT FROM STEREO"<<endl;
//			grabber = new StereoGrabber(640, 480, 15, enableRectification);
//			cout<<"frame dimension "<<grabber->width<<"x"<<grabber->height<<endl;
//			cout<<endl;
//		}
//	}
    /*else*/ if(video_input) {
		cout<<endl;
		cout<<"INPUT FROM VIDEO FILE "<< invideo <<endl;
		grabber = new FileGrabber(invideo.c_str());
		cout<<"frame dimension "<<grabber->width<<"x"<<grabber->height<<"\nFPS = "<<grabber->getFps()<<endl;
		cout<<endl;
	}
    else if(stereo_img_input) {
        cout << endl;
        cout << "INPUT FROM STEREO IMAGE SET" << endl;
        grabber = new imagestereograbber(stereo_img_directory, settingfilename, kinect_number, fps_image);
    } else if(stereo_kinect) {
        cout << endl;
        cout << "INPUT FROM KINECT N." << kinect_number << endl;
        grabber = new kinectgrabber(640, 480, kinect_number, settingfilename);
    }else if(image_input) {
		cout<<endl;
		cout<<"INPUT FROM IMAGE SET"<<endl;

		int index1 = imagesetfile.find_last_of(".");
#if __linux__
		int index2 = imagesetfile.find_last_of("/");
#else
		int index2 = imagesetfile.find_last_of("\\");
#endif
		if(index2 == string::npos) {
			index2 = -1;
		}
		if(index1 == string::npos) {
			cout<<"Usage: imbs -img <first image name>: input from a list of images\n"
				  "       <first image name> must be in the form of \"img%d.jpg\"\n"
				  "       where \"%d\" is the image number (e.g., \"img000.jpg\")\n"<<endl;
			exit(-1);
		}
		int d = 0;
		for(int i = index1 - 1; i > index2; i--) {
			if(isdigit(imagesetfile[i])) {
				d++;
			}
			else {
				break;
			}
		}
		if (d == 0) {
			cout<<"Usage: imbs -img <first image name>: input from a list of images\n"
				  "       <first image name> must be in the form of \"img%d.jpg\"\n"
				  "       where \"%d\" is the image number (e.g., \"img000.jpg\")\n"<<endl;
			exit(-1);
		}
		if(!fps_image_input) {
			cout<<"WARNING no -fps parameter set by user."<<endl;
		}
		grabber = new ImageGrabber(imagesetfile, d, fps_image);
		cout<<"frame dimension "<<grabber->width<<"x"<<grabber->height<<"\nFPS = "<<grabber->getFps()<<endl;
		cout<<endl;
	}
	else {
		cout<<endl;
		cout<<"INPUT FROM CAMERA"<<endl;
		grabber = new LiveGrabber(-1);
		cout<<"frame dimension "<<grabber->width<<"x"<<grabber->height<<endl;
		cout<<endl;
	}
	if(video_output) {
		cout<<endl;
		cout << "OUTPUT VIDEO SAVED IN " << outvideo << endl;
		//writer = cvCreateVideoWriter(outvideo.c_str(), CV_FOURCC('D', 'I', 'V', 'X'), /*fps_image*/10,
		writer = cvCreateVideoWriter(outvideo.c_str(), CV_FOURCC('M','J','P','G'), /*fps_image*/10,
				cvSize(mosaic_width, mosaic_height), 1);
		if(writer == NULL) {
			cout << "ERROR: UNABLE TO OPEN " << outvideo << endl;
		}
		else {
			cout << "OUTPUT VIDEO WRITER OPENED" << endl;
		}
	}

	//GUI WINDOWS
	if(show_gui) {
		cvInitFont(&font, CV_FONT_HERSHEY_SIMPLEX, 0.4, 0.4, 0, 1);

		frame = cvCreateImage( cvSize(grabber->getWidth(),grabber->getHeight()), IPL_DEPTH_8U, 3 );

		message = cvCreateImage( cvSize(grabber->getWidth(),grabber->getHeight()), IPL_DEPTH_8U, 3 );
		cvZero(message);
		cvPutText(message, "Creating", cvPoint(10,20), &font, CV_RGB(255, 255, 255));
		cvPutText(message, "initial", cvPoint(10,40), &font, CV_RGB(255, 255, 255));
		cvPutText(message, "background...", cvPoint(10,60), &font, CV_RGB(255, 255, 255));

		if(slow) {
			cout<<"SLOW MODE ENABLED\n";
		}
		cout<<"press 'ESC' for exiting"<<endl;
		cout<<"      'f' for NORMAL MODE"<<endl;
		cout<<"      's' for SLOW MODE again"<<endl;
		cout<<"      any other key for next frame"<<endl;
	}

	cout<<endl;
	cout<<"FOREGROUND THRESHOLD: "<<fgThreshold<<endl;
	cout<<"BIN WIDTH: "<<binWidth<<endl;
	cout<<"SAMPLING PERIOD: "<<samplingPeriod<<" ms"<<endl;
	cout<<"MINIMAL BIN HEIGHT: "<<minBinHeight<<endl;
	cout<<"NUMBER OF SAMPLES: "<<numSamples<<endl;
	cout<<"ALPHA: "<<alpha<<endl;
	cout<<"BETA: "<<beta<<endl;
	cout<<"TAU_S: "<<tau_s<<endl;
	cout<<"TAU_H: "<<tau_h<<endl;
	cout<<"AREA THRESHOLD: "<<area<<endl;
	cout<<endl;

	//IMBS
	IMBS* imbsStereo = new IMBS(grabber,
                fgThreshold,
				binWidth,
				samplingPeriod,
				totalImageParts,
				minBinHeight,
				numSamples);

	IMBS* imbs = new IMBS(grabber,
				fgThreshold,
				binWidth,
				samplingPeriod,
				totalImageParts,
				minBinHeight,
				numSamples,
				alpha,
				beta,
				tau_s,
				tau_h,
				area,
				imbsStereo);

//	IplImage *tempStereo = cvCreateImage(cvSize(grabber->width, grabber->height), IPL_DEPTH_8U, 1);
//	cvZero(0);
    //cvNamedWindow("Frame", 1);
//    cvNamedWindow("Depth", 1);
	if(show_gui) {

        //cvSetMouseCallback("Frame", onMouse, imbs);
        //cvSetMouseCallback("Depth", onMouse, imbs);
//		cvSetMouseCallback("background", onMouse, imbs);

		//cvSetMouseCallback("FG", onMouse, imbs);
	}

	//LOAD INITIAL BACKGROUND
	if(load_bg){
		cout<<"loading initial background...";
		cout.flush();
		if(imbs->loadBackground(load_bg_filename.c_str())) {
			cout<<"done"<<endl;
		}
		else {
			cerr << "Unable to open file "<<load_bg_filename<<endl;
			cerr<<"Process terminated."<<endl;
			exit(-1);
		}
	}
	//SAVE INITIAL BACKGROUND
	if(save_bg) {
		cout<<"Initial background will be saved as "<<save_bg_filename<<endl;
		imbs->setSaveBackground(&save_bg_filename);
	}
	//SAVE FOREGROUND IMAGES
	if(save_fg) {
		createDirectory(save_fg_path);
	}
	//LOAD INITIAL STEREO BACKGROUND
	if(load_stereo_bg){
		cout<<"loading initial stereo background...";
		cout.flush();
		if(imbsStereo->loadStereoBackground(load_stereo_bg_filename.c_str())) {
			cout<<"done"<<endl;
		}
		else {
			cerr << "Unable to open file "<<load_stereo_bg_filename<<endl;
			cerr<<"Process terminated."<<endl;
			return EXIT_FAILURE;
		}
	}
	//SAVE INITIAL STEREO BACKGROUND
	if(save_stereo_bg) {
		cout<<"Initial stereo background will be saved as "<<save_stereo_bg_filename<<endl;
        imbsStereo->setSaveStereoBackground(save_stereo_bg_filename);
	}

	if(verbose) {
		cout<<"starting main cicle"<<endl;
	}

	unsigned long t1 = getTime();
	int counter_fps = 0;

    if(!load_stereo_bg) {
        cout << "Creating Stereo Background...";
        cout.flush();

        while(runStereo) {
        	        	
            if(!grabber->getData()) {
            	cerr << "Unable to get stereo data" << endl;
            	return EXIT_FAILURE;
            }

            imbsStereo->processStereo();

            if(imbsStereo->getBgModel()[0].isValid[0]) {
                runStereo = false;
            }
        }
        cout << "done" << endl;

        grabber->reset();

    }//if !load_stereo_bg
    
    cout << "Process started." << endl;

	while (run) {

		if(!grabber->getData()){
			unsigned long t2 = getTime();
            cout<<"final time = "<<t2<<" initial time = "<<t1<<endl;
			cout<<"elapse time = "<<t2-t1<<" ms"<<endl;
			cout<<"num frames = "<<counter_fps<<endl;
			cout<<"fps = "<<(double)(counter_fps / (((double)(t2-t1))/1000))<<endl;
			run = false;
			continue;
		}


		if(show_gui) {
			cvCopy(grabber->frame, frame);
		}

		if(verbose) {
			cout<<"data grabbed"<<endl;
		}

		if(lastTimeout > 0 && lastTimeout == grabber->getTimestamp()) {
			cout<<"ERROR: lastTimeout ("<<lastTimeout<<") = grabber->timestamp (";
			cout<<grabber->getTimestamp()<<")\n";
			abort();
		}
		else
			lastTimeout = grabber->getTimestamp();

		if(verbose) {
			cout<<"start executing IMBS...";
			cout.flush();
		}

		//IMBS PROCESS
		imbs->process();

		counter_fps++;

		if(verbose) {
			cout<<"done"<<endl;
		}

		//DISPLAYING
		if(show_gui) {
			//FRAME NUMBER
			string s;
			stringstream out;
			out << grabber->frameNumber;
			s.assign(out.str());			
			int s_size = s.size();
			cvRectangle(frame, cvPoint(10, 5), cvPoint(8 + s_size * 10, 16), cvScalar(255, 255, 255), -1);
			cvPutText(frame, s.c_str(), cvPoint(10,15), &font, CV_RGB(0, 0, 0));

			//TIMESTAMP
			double formatted_time = grabber->getTimestamp()/1000.0;

			int hours = (int)floor(formatted_time/3600);
			double minutes_d = (formatted_time/3600 - hours) * 60;
			int minutes = (int)minutes_d;
			double seconds_d = (minutes_d*60 - minutes*60);
			int seconds = (int)seconds_d;
			double milliseconds_d = (seconds_d*1000 - seconds*1000);
			int milliseconds = (int)milliseconds_d;

			string s2;
			stringstream out2;
			if(hours < 10) {
				out2 << "0";
			}
			out2 << hours << ":";
			if(minutes < 10) {
				out2 << "0";
			}
			out2 << minutes << ":";
			if(seconds < 10) {
				out2 << "0";
			}
			out2 << seconds << ":";
			if(milliseconds < 10) {
				out2 << "00";
			}
			else if(milliseconds < 100) {
				out2 << "0";
			}
			out2 << milliseconds;
			s2.assign(out2.str());
			int s_size2 = s2.size();
			cvRectangle(frame, cvPoint(100, 5), cvPoint(98 + s_size2 * 10, 16),
					cvScalar(255, 255, 255), -1);
			cvPutText(frame, s2.c_str(), cvPoint(100,15), &font, CV_RGB(0, 0, 0));

			images[0] = frame;
			//cvShowImage("Frame", frame);

			if(!imbs->getBgModel()[0].isValid[0]) {
				images[1] = message;
				images[2] = message;
                images[3] = message;
			}
			else {
                images[1] = imbs->bgImage;
                //images[2] = imbs->fgFilteredImage;//imbs->fgImage;
                images[2] = imbs->fgColoredImage;//imbs->fgImage;
                images[3] = imbs->fgFilteredImage;
                //images[3] = imbs->depthBlobs;
            }

			if(mosaic) {
				cvReleaseImage(&mosaic);
            }
            mosaic = imageMosaic(2, 2, mosaic_width, mosaic_height, images);
            cvShowImage("PLT", mosaic);

			int keyboard = -1;
			if(slow) {
				keyboard = cvWaitKey(0);
				if((char)keyboard == 'f') {
					cout<<"NORMAL MODE\n";
					slow = false;
				}
				else if((char)keyboard == 27) {
						cout<<"quitting...\n";
						run = false;
						startScene3d = false;
				}
			}
			else {
				keyboard = cvWaitKey(30);
				if((char)keyboard == 's') {
					cout<<"SLOW MODE\n";
					slow = true;
				}
				else if((char)keyboard == 27) {
					cout<<"quitting...\n";
					run = false;
					startScene3d = false;
				}
			}

		}//if show_gui

		if(save_fg) {

#ifdef __linux__
			if(save_fg_path.find_last_of("/") != (save_fg_path.size() - 1)) {
				save_fg_path.append("/");
			}
#else
			if(save_fg_path.find_last_of("\\") != (save_fg_path.size() - 1)) {
				save_fg_path.append("\\");
			}
#endif
			string s(save_fg_path);
			s.append("fg_");

			if(image_input) {
				ImageGrabber* ig = (ImageGrabber*) grabber;
				string ss = ig->getImagePath();
#ifdef __linux__
				int idx = ss.find_last_of("/");
#else
				int idx = ss.find_last_of("\\");
#endif
				s.append(ss.substr(idx+1));
			}
			if(video_input) {
				stringstream out;
				out << grabber->frameNumber;
				s.append(out.str());
				s.append(".png");
			}
			cout<<"saving foreground image as "<<s<<endl;
			cvSaveImage(s.c_str(), imbs->fgImage);
		}

		if(video_output) {
			cvWriteFrame(writer, mosaic);
		}

	}//end while run



	if(show_gui) {
		cvDestroyAllWindows();
	}

	if(video_output) {
		cvReleaseVideoWriter(&writer);
	}

}

void onMouse(int event, int x, int y, int flag, void *param) {
//	IplImage* greenFieldCopy = cvCreateImage(cvSize(640, 480), IPL_DEPTH_8U, 3);
//	cvCopy(greenField, greenFieldCopy);
	IMBS *imbs = (IMBS *)param;
	switch (event) {
		case CV_EVENT_LBUTTONDOWN:
		{
			cout<<"--------------------------------------------------"<<endl;

			cout << "Image: " << std::endl;

			std::cout << "x: "<<x<<"  y: "<<y<<endl;

            CvScalar tempPixel = cvGet2D(grabber->frame, y, x);

            std::cout << "r: " << tempPixel.val[2] << "  g: " << tempPixel.val[1] << "  b: " << tempPixel.val[0] << std::endl;

            /*double X, Y, Z;
            static_cast<imagestereograbber*>(imbs->grabber)->get3DPoint(x,y,&X,&Z,&Y);

            std::cout << "X: " << X << " Y: " << Y << " Z: " << Z << std::endl;*/

            Grabber *grabber = imbs->grabber;

            std::cout << "Depth: " << std::endl;
            double wx, wy, wz;
            static_cast<imagestereograbber*>(grabber)->get3DPoint(x, y, &wx, &wz, &wy);
            std::cout << "X: " << wx << " Y: " << wy << " Z: " << wz << std::endl;

            cv::Mat depthMap = grabber->getDepth16bit();

			CBlobResult blobs = CBlobResult( imbs->fgFilteredImage, NULL, 0 );
			CBlob currentBlob;
			int numBlobs = blobs.GetNumBlobs();

                for(int k = 0; k < numBlobs; k++) {
                    currentBlob = blobs.GetBlob(k);

                    int max_x = (int)currentBlob.MaxX();
                    int min_x = (int)currentBlob.MinX();
                    int max_y = (int)currentBlob.MaxY();
                    int min_y = (int)currentBlob.MinY();


                    int width = max_x - min_x;
                    int height = max_y - min_y;

                    std::cout << "h: " << height << " width: " << width << std::endl;

                    std::cout << "max_x: " << max_x << " min_x " << min_x << " max_y " << max_y << " min_y " << min_y << std::endl;

                    if(height > 0 && width > 0 && x <= max_x && x >= min_x && y <=max_y && y >= min_y) {

                    	cout << "Blob:" << endl;
                    	cout << "max_x: " << max_x << "  min_x: " << min_x << endl;
                    	cout << "max_y: " << max_y << "  min_y: " << min_y << endl;

                    	int cnt_i = 0;
                    	int cnt_j = 0;
                    	//int sumY = 0;

                    	vector<double> Yvalues;
                        for(int i = min_y; i < max_y; ++i) {

                        	if(cnt_i > 20)
                        		break;

                        	int isvalid = false;
                            for(int j = min_x; j < max_x; ++j) {
                                if((uchar)imbs->fgFilteredImage->imageData[j+640 * (i % 480)] == 255) {
                                	if(depthMap.at<ushort>(i,j) > 500 && depthMap.at<ushort>(i,j) < 4500) {
                                		double X, Y, Z;
                                        static_cast<imagestereograbber*>(grabber)->get3DPoint(j,i,&X,&Z,&Y);

                                		//cout << "Y: " << Y << endl;

                                		//sumY += Y;

                                		Yvalues.push_back(Y);

                                		cnt_j++;
                                	}
                                	isvalid = true;
                                }
                            }
                            if(isvalid)
                            	cnt_i++;
                        }

                        if(cnt_j == 0 || Yvalues.empty()) {
                        	std::cout << "cnt_j = 0" << std::endl;
                        	//cvRectangle(fgFilteredImage, cvPoint(min_x,min_y), cvPoint(max_x,max_y), CV_RGB(0,0,0), -1);
                        }
                        else {

                        	std::sort(Yvalues.begin(), Yvalues.end(), sort_using_greater_than);

                        	int n = std::min(10, (int)Yvalues.size());

                        	double sumY = 0;

                        	for(std::vector<double>::size_type index = 0; index < n; ++index)
                        		sumY += Yvalues.at(index);


                        	double altezza = sumY / n;
                        	std::cout << "Altezza: " << altezza << std::endl;
                        }

                    }
                }

//			int plan_view_x = -1;
//			int plan_view_y = -1;
//
//			int plan_view_topLeft_x = -1;
//			int plan_view_topLeft_y = -1;
//			int plan_view_topLeft_z = -1;
//
//			if(adjustedPts3D[x + 640 * ( y % 480)].A != 0) {
//				plan_view_topLeft_x = (adjustedPts3D[x + 640 * ( y % 480)].X + planViewTranslation) * planViewScale;
//				plan_view_topLeft_y = (adjustedPts3D[x + 640 * ( y % 480)].Y + planViewTranslation) * planViewScale;
//				plan_view_topLeft_z = (adjustedPts3D[x + 640 * ( y % 480)].Z + planViewTranslation) * planViewScale;
////
////				plan_view_x = plan_view_topLeft_x;
////				plan_view_y = plan_view_topLeft_y;
////
//				cout << "plan_view_x: " << plan_view_topLeft_x << " plan_view_y: " << plan_view_topLeft_y  <<
//						" plan_view_z: " << plan_view_topLeft_z << endl;

//				cout << "x: " << adjustedPts3D[x + 640 * ( y % 480)].X << " y: " <<
//						adjustedPts3D[x + 640 * ( y % 480)].Y << " z: " <<
//						adjustedPts3D[x + 640 * ( y % 480)].Z << endl;
////
////				cvCircle(greenField,cvPoint(plan_view_x,plan_view_y), 5,  cvScalar(0, 0, 255), -1);
//			}
//			cout << x + 640 * ( y % 480) << endl;
			//short *d = ->getDisparity();
//			if(grabber->disparity != NULL)
//				cout << "disparity: " << grabber->disparity[x + 640 * ( y % 480)] << endl;
//			else
//				cout << "Error" << endl;
//
//			cvShowImage("PlanView",greenFieldCopy);

/*			IplImage* frame = imbs->getFrame();

			int index = (x*frame->nChannels) + (y*(frame->widthStep/sizeof(uchar)));

			int hsv_index = (x*imbs->hsvFrame->nChannels) + (y*(imbs->hsvFrame->widthStep/sizeof(float)));

			int r = (uchar)frame->imageData[index + 2];
			int g = (uchar)frame->imageData[index + 1];
			int b = (uchar)frame->imageData[index];

			cout<<"R: "<<r<<" G: "<<g<<" B: "<<b<<endl;

			IMBS::BgModel* bm = imbs->getBgModel();
			for(int e = 0; e < imbs->getMaxBgValues(); e++) {
				if(!bm[index/3].isValid[e]) {
					cout<<endl;
					break;
				}
				cout<<"(";
				cout<<(int)bm[index/3].values[e].r<<" ";
				cout<<(int)bm[index/3].values[e].g<<" ";
				cout<<(int)bm[index/3].values[e].b<<" ";
				cout<<"["<<(int)bm[index/3].counter[e]<<"])";
				if(bm[index/3].isForeground[e]) {
					cout<<"(F)";
				}
				cout<<" ";
			}

			bool isForeground = true;
			for(int n = 0; n < imbs->getMaxBgValues(); n++) {
				if(!bm[index/3].isValid[n]) {
					if(n == 0) {
						isForeground = false;
					}
					break;
				}
				else {
					int d = max(abs(bm[index/3].values[n].b - (uchar)frame->imageData[index]),
									abs(bm[index/3].values[n].g - (uchar)frame->imageData[index+1]) );
					d = max(d, abs(bm[index/3].values[n].r - (uchar)frame->imageData[index+2]) );
					//cout << "d = " << d << endl;
					if(d < imbs->fgThreshold){
						if(bm[index/3].isForeground[n]) {
							isForeground = true;
							cout<<"Conditional Update -- ";
							break;
						}
						else {
							isForeground = false;
						}
					}
				}
			}
			if(isForeground) {
				cout<<"IS foreground"<<endl;
			}
			else {
				cout<<"NO foreground"<<endl;
				cout<<"foreground value "<<(int)((uchar)imbs->fgImage->imageData[index/3])<<endl;
				return;
			}

			float h_i, s_i, v_i;
			float h_b, s_b, v_b;
			float h_diff, s_diff, v_ratio;

			IplImage* hsvMat = cvCreateImage(cvSize(1,1), IPL_DEPTH_32F, 3);
			IplImage* rgbMat = cvCreateImage(cvSize(1,1), IPL_DEPTH_32F, 3);
			bool isShadow = false;
			bool isLight = false;

			for(int n = 0; n < imbs->getMaxBgValues(); n++) {
				if(!bm[index/3].isValid[n]) {
					break;
				}
				if(bm[index/3].isForeground[n]) {
					continue;
				}

				float f_b = ((uchar)bm[index/3].values[n].b)/255.f;
				float f_g = ((uchar)bm[index/3].values[n].g)/255.f;
				float f_r = ((uchar)bm[index/3].values[n].r)/255.f;

				float * data = (float *)rgbMat->imageData;
				data[0] = f_b;
				data[1] = f_g;
				data[2] = f_r;

				cvCvtColor(rgbMat, hsvMat, CV_BGR2HSV);

				data = (float *)hsvMat->imageData;
				h_b = data[0];
				s_b = data[1];
				v_b = data[2];

				data = (float *)imbs->hsvFrame->imageData;
				v_i = data[hsv_index + 2];
				s_i = data[hsv_index + 1];
				h_i = data[hsv_index];

				v_ratio = v_i / v_b;
				s_diff = abs(s_i - s_b);
				h_diff = abs(h_i - h_b);

				cout<<"H_I: "<<h_i<<"  H_B: "<<h_b<<endl;
				cout<<"S_I: "<<s_i<<"  S_B: "<<s_b<<endl;
				cout<<"V_I: "<<v_i<<"  V_B: "<<v_b<<endl;
				cout<<"h diff = "<<(h_i - h_b)<<endl;
				cout<<"s diff = "<<(s_i - s_b)<<endl;
				cout<<"v ratio = "<<v_ratio<<endl;

				if( v_ratio >= imbs->alpha && v_ratio <= imbs->beta && s_diff <= imbs->tau_s && h_diff <= imbs->tau_h ) {
					isShadow = true;
					break;
				}

				else if( v_ratio >= imbs->beta && v_ratio <= 1.15 &&
						s_diff < 0.1 && h_diff <= imbs->tau_h) {
					isLight = true;
					break;
				}
			}
			cvReleaseImage(&hsvMat);
			cvReleaseImage(&rgbMat);

			if(isShadow) {
				cout<<"IS shadow"<<endl;
			}
			else {
				cout<<"NO shadow"<<endl;
			}
			if(isLight) {
				cout<<"IS LIGHT"<<endl;
			}
			else {
				cout<<"NO LIGHT"<<endl;
			}

			cout<<"foreground value "<<(int)((uchar)imbs->fgImage->imageData[index/3])<<endl;*/

		}
		case CV_EVENT_RBUTTONDOWN:
		{
   		   	break;
		}
	}//end switch
}//end onmouse

unsigned long getTime()
{
#ifdef __linux__
	struct timeval tv;
	gettimeofday(&tv,0);
	return (unsigned long)(tv.tv_sec%86400)*1000+(unsigned long)tv.tv_usec/1000;
#else
	struct timeb timebuffer;
	ftime( &timebuffer );
	unsigned short	millitm = timebuffer.millitm;
	time_t sectm = timebuffer.time;
	return (unsigned long)(sectm%86400)*1000+(unsigned long)millitm;
#endif
}

void createDirectory(string name) {
	DIR *dh = opendir(name.c_str());
	if(!dh){
		int m;
#ifdef __linux__
		m = mkdir(name.c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
#else
		m = mkdir(name.c_str());
#endif
		if(m == 0){
			cout<<name<<" folder created."<<endl;
		}
		else{
			cout<<"unable to create "<<name<<" folder."<<endl;
			cout<<"aborting..."<<endl;
			abort();
		}
	}
	closedir(dh);
}


//POSITION PLAYERS FILES
//bool loadPositionPlayersFile(const char *cfg, vector<previousPosition>& positions) {
//	printf("Opening position players cfg %s\n", cfg);
//	ifstream configfile(cfg);
//	if(!configfile) {
//		return false;
//	}
//	int i = 0;
//	previousPosition tempPlayer;

//	while (!configfile.eof() && i < 3) {
//		configfile >> tempPlayer.role >> tempPlayer.pos.x >> tempPlayer.pos.y;
//		positions.push_back(tempPlayer);
//		++i;
//	}

//	configfile.close();
//	return true;
//}

//POSITION FILE READER
bool loadPositionFile(const char *cfg, double* degreesX, double* degreesZ, double* Yoffset, double* planViewTranslation, double *planViewScale){
	printf("Opening position cfg %s\n", cfg);
	ifstream configfile(cfg);
 	if (!configfile) {
 		return false;
 	}

	char buffer[50];
	bool no_correction = true;
	configfile >> buffer;

 	while (!configfile.eof())
 	{
 		//printf("buffer -%s-\n", buffer);

 		//printf("buffer -%s-\n", buffer);

 		if (strcmp(buffer,"[degreesX]")==0){
	 		configfile >> buffer;
	 		*degreesX = atof(buffer);
	 		printf("degreesX = %f\n", *degreesX);
	 		no_correction = false;
	 		configfile >> buffer;
		}
 		else  if (strcmp(buffer,"[degreesZ]")==0){
			configfile >> buffer;
	 		*degreesZ = atof(buffer);
	 		printf("degreesZ = %f\n", *degreesZ);
	 		no_correction = false;
	 		configfile >> buffer;
		}
		else  if (strcmp(buffer,"[Yoffset]")==0){
			configfile >> buffer;
	 		*Yoffset = atof(buffer);
	 		printf("Yoffset = %f\n", *Yoffset);
	 		no_correction = false;
	 		configfile >> buffer;
		}
		else  if (strcmp(buffer,"[planViewTranslation]")==0){
			configfile >> buffer;
	 		*planViewTranslation = atof(buffer);
	 		printf("planViewTranslation = %f\n", *planViewTranslation);
	 		no_correction = false;
	 		configfile >> buffer;
		}
		else  if (strcmp(buffer,"[planViewScale]")==0){
			configfile >> buffer;
	 		*planViewScale = atof(buffer);
	 		printf("planViewScale = %f\n", *planViewScale);
	 		no_correction = false;
	 		configfile >> buffer;
		}
 		else {
			cerr << "Position file error" << endl;
			return false;
		}
 	}
 	if(no_correction)
	{
		printf("\n          ******************         \n");
		printf("WARNING: no correction made on camera coordinates\n");
		printf("          ******************         \n\n");
	}
 	configfile.close();
 	return true;
}


IplImage* imageMosaic(int rows, int cols, int width, int height, IplImage** images) {
	IplImage* mosaic = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 3);
	cvZero(mosaic);
	int image_width = width / cols;
	int image_height = height / rows;
	int x = 0;
	int y = 0;
	int mosaic_step = mosaic->widthStep/sizeof(uchar);
	uchar* mosaic_data = (uchar *)mosaic->imageData;
	int mosaic_channels = mosaic->nChannels;
	for(int r = 0; r < rows; r++, y+=image_height) {
		x = 0;
		for(int c = 0; c < cols; c++, x+=image_width) {
			IplImage* image = images[r*cols + c];

			IplImage* r_image =
					cvCreateImage(cvSize(image_width, image_height), image->depth, image->nChannels);

			cvResize(image, r_image, CV_INTER_CUBIC);

			int image_step = r_image->widthStep/sizeof(uchar);
			uchar* image_data = (uchar *)r_image->imageData;
			int image_channels = r_image->nChannels;

			for(int i = y, ii = 0; ii < image_height; i++, ii++) {
				for(int j = x, jj = 0; jj < image_width; j++, jj++) {

					for(int k = 0; k < mosaic_channels; k++) {
						if(image_channels == mosaic_channels) {
							mosaic_data[i*mosaic_step+j*mosaic_channels+k] =
									image_data[ii*image_step+jj*image_channels+k];
						}
						else {
							mosaic_data[i*mosaic_step+j*mosaic_channels+k] =
									image_data[ii*image_step+jj];
						}
					}
				}
			}
			cvReleaseImage(&r_image);
		}
	}
	return mosaic;
}
