//
// file: svsglwin.h
//

#ifndef _SVSGLWIN_H_
#define _SVSGLWIN_H_

#include <FL/Fl_Gl_Window.H>
#include <FL/gl.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include "svsclass.h"


#define MAX_POINTS 1288*1032  // max. number of 3d points possible

//extern void Display3D(svsStereoImage *di);

extern void Display3D(svsStereoImage *di, svs3Dpoint* globalPts3D);

class SvsGLWindow : public Fl_Gl_Window {
  
  //
  // overview: SvsGLWindow is a (FLTK widget) window which has
  //           complete OpenGL capabilities.  
  //           Currently, this class provides functions for taking a 
  //           disparity image and displaying it as a 3D (point cloud) 
  //           image (using the original 2D image to color the points).  
  //           It also provides basic navigation around the 3D space created 
  //           (rotation, translation, and scaling).  The (x,y,z) axes can also
  //           be turned on/off.
  //


 private:

  void draw();              // draw function called when necessary by FLTK
  int handle(int);          // user event handling function, called by FLTK

  void recalcModelView();   // recalculate 3D view
  
  
  // this finds average Z over a center pixel neighborhood
  //bool calcCenterZ(svsStereoImage *di, int nh);

	bool calcCenterZ(svsStereoImage *di, int nh, svs3Dpoint* globalPts3D);


  int moving, beginx, beginy, numPoints, newModel, axes;

  GLfloat anglex, angley, anglez;      // 3D view paramaters
  GLfloat scale;
  GLfloat xshift,  yshift, zshift;
  
  float centerx, centery, centerz;     // parameters to make viewing more intuitive
  float minx, miny, minz; 
  float maxx, maxy, maxz;

  int bufsize;
  GLfloat *colorListR;       // point colors	
  GLfloat *colorListG;       // point colors
  GLfloat *colorListB;       // point colors
  GLfloat *pointListX, *pointListY, *pointListZ;     // points 


 public:
  
  float max, min;		// not using these right now...
  float scaleFactor;		// how much to change scale

  SvsGLWindow(int X, int Y, int W, int H);  // constructor

  // set up 3D image; new_points is false for changing color
  //void DisplayDisparityImage(svsStereoImage *di);
  
  void DisplayDisparityImage(svsStereoImage *di, svs3Dpoint* globalPts3D);

  void scaleImage(float value);    // scales the image with a fixed aspect ratio

  // (these are dumb names because the "horizontal" changes depending on rotation;
  //  it should just be along x, y, and z)

  void transImageH(float value);   // translate the 3D image horizontally
  void transImageV(float value);   // translate the 3D image vertically
  void transImageZ(float value);   // translate the 3D image depth-wise
  void rotImageH(float value);     // rotate the 3D image horizontally
  void rotImageV(float value);     // rotate the 3D image vertically
  void rotImageZ(float value);     // rotate the 3D image depth-wise

  void setDisplayAxes(int value);  // display or don't display the axes (value = 1 or 0)
};

#endif  // _SVSGLWIN_H_
